@@ -77,6 +77,11 @@ struct App
     } apply_configuration_state;
 };
 
+/* Response codes for custom buttons in the main dialog */
+enum {
+    RESPONSE_MAKE_DEFAULT = 1
+};
+
 static void rebuild_gui (App *app);
 static void on_clone_changed (GtkWidget *box, gpointer data);
 static void on_rate_changed (GtkComboBox *box, gpointer data);
@@ -516,13 +521,13 @@ rebuild_current_monitor_label (App *app)
 	{
 	    GdkColor black = { 0, 0, 0, 0 };
 
-	    gtk_widget_modify_bg (app->current_monitor_event_box, app->current_monitor_event_box->state, &color);
+	    gtk_widget_modify_bg (app->current_monitor_event_box, gtk_widget_get_state (app->current_monitor_event_box), &color);
 
 	    /* Make the label explicitly black.  We don't want it to follow the
 	     * theme's colors, since the label is always shown against a light
 	     * pastel background.  See bgo#556050
 	     */
-	    gtk_widget_modify_fg (app->current_monitor_label, GTK_WIDGET_STATE (app->current_monitor_label), &black);
+	    gtk_widget_modify_fg (app->current_monitor_label, gtk_widget_get_state (app->current_monitor_label), &black);
 	}
 	else
 	{
@@ -1654,15 +1659,17 @@ paint_background (FooScrollArea *area,
 {
     GdkRectangle viewport;
     GtkWidget *widget;
+    GtkStyle *widget_style;
 
     widget = GTK_WIDGET (area);
 
     foo_scroll_area_get_viewport (area, &viewport);
+    widget_style = gtk_widget_get_style (widget);
 
     cairo_set_source_rgb (cr,
-                          widget->style->base[GTK_STATE_SELECTED].red / 65535.0,
-                          widget->style->base[GTK_STATE_SELECTED].green / 65535.0,
-                          widget->style->base[GTK_STATE_SELECTED].blue / 65535.0);
+                          widget_style->base[GTK_STATE_SELECTED].red / 65535.0,
+                          widget_style->base[GTK_STATE_SELECTED].green / 65535.0,
+                          widget_style->base[GTK_STATE_SELECTED].blue / 65535.0);
 
     cairo_rectangle (cr,
 		     viewport.x, viewport.y,
@@ -1673,9 +1680,9 @@ paint_background (FooScrollArea *area,
     foo_scroll_area_add_input_from_fill (area, cr, on_canvas_event, NULL);
 
     cairo_set_source_rgb (cr,
-                          widget->style->dark[GTK_STATE_SELECTED].red / 65535.0,
-                          widget->style->dark[GTK_STATE_SELECTED].green / 65535.0,
-                          widget->style->dark[GTK_STATE_SELECTED].blue / 65535.0);
+                          widget_style->dark[GTK_STATE_SELECTED].red / 65535.0,
+                          widget_style->dark[GTK_STATE_SELECTED].green / 65535.0,
+                          widget_style->dark[GTK_STATE_SELECTED].blue / 65535.0);
 
     cairo_stroke (cr);
 }
@@ -2043,10 +2050,10 @@ apply_configuration_returned_cb (DBusGProxy       *proxy,
     gtk_widget_set_sensitive (app->dialog, TRUE);
 }
 
-static void
-apply (App *app)
+static gboolean
+sanitize_and_save_configuration (App *app)
 {
-    GError *error = NULL;
+    GError *error;
 
     gnome_rr_config_sanitize (app->current_configuration);
 
@@ -2056,13 +2063,25 @@ apply (App *app)
 
     ensure_current_configuration_is_saved ();
 
+    error = NULL;
     if (!gnome_rr_config_save (app->current_configuration, &error))
     {
 	error_message (app, _("Could not save the monitor configuration"), error->message);
 	g_error_free (error);
-	return;
+	return FALSE;
     }
 
+    return TRUE;
+}
+
+static void
+apply (App *app)
+{
+    GError *error = NULL;
+
+    if (!sanitize_and_save_configuration (app))
+	return;
+
     g_assert (app->connection == NULL);
     g_assert (app->proxy == NULL);
     g_assert (app->proxy_call == NULL);
@@ -2234,8 +2253,8 @@ get_output_for_window (GnomeRRConfig *configuration, GdkWindow *window)
 static void
 select_current_output_from_dialog_position (App *app)
 {
-    if (GTK_WIDGET_REALIZED (app->dialog))
-	app->current_output = get_output_for_window (app->current_configuration, app->dialog->window);
+    if (gtk_widget_get_realized (app->dialog))
+	app->current_output = get_output_for_window (app->current_configuration, gtk_widget_get_window (app->dialog));
     else
 	app->current_output = NULL;
 
@@ -2294,6 +2313,61 @@ apply_button_clicked_cb (GtkButton *button, gpointer data)
     app->apply_button_clicked_timestamp = gtk_get_current_event_time ();
 }
 
+static void
+success_dialog_for_make_default (App *app)
+{
+    GtkWidget *dialog;
+
+    dialog = gtk_message_dialog_new (GTK_WINDOW (app->dialog),
+				     GTK_DIALOG_MODAL,
+				     GTK_MESSAGE_INFO,
+				     GTK_BUTTONS_OK,
+				     _("The monitor configuration has been saved"));
+    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+					      _("This configuration will be used the next time someone logs in."));
+
+    gtk_dialog_run (GTK_DIALOG (dialog));
+    gtk_widget_destroy (dialog);
+}
+
+static void
+make_default (App *app)
+{
+    char *command_line;
+    char *source_filename;
+    char *dest_filename;
+    char *dest_basename;
+    GError *error;
+
+    if (!sanitize_and_save_configuration (app))
+	return;
+
+    dest_filename = gconf_client_get_string (app->client, "/apps/gnome_settings_daemon/xrandr/default_configuration_file", NULL);
+    if (!dest_filename)
+	return; /* FIXME: present an error? */
+
+    dest_basename = g_path_get_basename (dest_filename);
+
+    source_filename = gnome_rr_config_get_intended_filename ();
+
+    command_line = g_strdup_printf ("pkexec %s/gnome-display-properties-install-systemwide %s %s",
+				    SBINDIR,
+				    source_filename,
+				    dest_basename);
+
+    error = NULL;
+    /* FIXME: pick up stderr and present it nicely in case of error */
+    if (!g_spawn_command_line_sync (command_line, NULL, NULL, NULL, &error))
+	error_message (app, _("Could not set the default configuration for monitors"), error ? error->message : NULL);
+    else
+	success_dialog_for_make_default (app);
+
+    g_free (dest_filename);
+    g_free (dest_basename);
+    g_free (source_filename);
+    g_free (command_line);
+}
+
 static GtkWidget*
 _gtk_builder_get_widget (GtkBuilder *builder, const gchar *name)
 {
@@ -2338,9 +2412,9 @@ run_application (App *app)
     g_signal_connect_after (app->dialog, "map-event",
 			    G_CALLBACK (dialog_map_event_cb), app);
 
-    gtk_window_set_default_icon_name ("gnome-display-properties");
+    gtk_window_set_default_icon_name ("preferences-desktop-display");
     gtk_window_set_icon_name (GTK_WINDOW (app->dialog),
-			      "gnome-display-properties");
+			      "preferences-desktop-display");
 
     app->current_monitor_event_box = _gtk_builder_get_widget (builder,
     						   "current_monitor_event_box");
@@ -2445,6 +2519,11 @@ restart:
 	apply (app);
 	goto restart;
 	break;
+
+    case RESPONSE_MAKE_DEFAULT:
+	make_default (app);
+	goto restart;
+	break;
     }
 
     gtk_widget_destroy (app->dialog);
